import pytest
import asyncio


class TestSync:
    def test_basic_mainloop(self, sync_loop):
        async def foo():
            return "bar"

        async def bar():
            return "baz"

        res = sync_loop.run_until_complete(foo())
        assert res == "bar"
        res = sync_loop.run_until_complete(bar())
        assert res == "baz"

    def test_explicit_mainloop(self):
        async def foo():
            return "bar"

        async def bar():
            return "baz"

        loop = asyncio.new_event_loop()
        res = loop.run_until_complete(foo())
        assert res == "bar"
        res = loop.run_until_complete(bar())
        assert res == "baz"
        loop.close()

    def test_basic_errloop(self, sync_loop):
        async def foo():
            raise RuntimeError("bar")

        with pytest.raises(RuntimeError) as res:
            sync_loop.run_until_complete(foo())
        if res.value.args[0] != "bar":
            raise res.value

    def test_explicit_errloop(self):
        async def foo():
            raise RuntimeError("bar")

        loop = asyncio.new_event_loop()
        with loop:
            with pytest.raises(RuntimeError) as res:
                try:
                    loop.run_until_complete(foo())
                finally:
                    pass
            if res.value.args[0] != "bar":
                raise res.value



"""Tests support for new syntax introduced by PEP 492."""

import sys
import types
import unittest
import pytest

try:
    from test import support
except ImportError:
    from asyncio import test_support as support
from unittest import mock

import asyncio
from .. import utils as test_utils


class BaseTest(test_utils.TestCase):
    def setUp(self):
        super().setUp()
        self.loop = asyncio.BaseEventLoop()
        self.loop._process_events = mock.Mock()
        self.loop._selector = mock.Mock()
        self.loop._selector.select.return_value = ()
        self.set_event_loop(self.loop)


class LockTests(BaseTest):
    def test_context_manager_async_with(self):
        primitives = [
            asyncio.Lock(loop=self.loop),
            asyncio.Condition(loop=self.loop),
            asyncio.Semaphore(loop=self.loop),
            asyncio.BoundedSemaphore(loop=self.loop),
        ]

        async def test(lock):
            await asyncio.sleep(0.01, loop=self.loop)
            self.assertFalse(lock.locked())
            async with lock as _lock:
                self.assertIs(_lock, None)
                self.assertTrue(lock.locked())
                await asyncio.sleep(0.01, loop=self.loop)
                self.assertTrue(lock.locked())
            self.assertFalse(lock.locked())

        for primitive in primitives:
            self.loop.run_until_complete(test(primitive))
            self.assertFalse(primitive.locked())

    def test_context_manager_with_await(self):
        primitives = [
            asyncio.Lock(loop=self.loop),
            asyncio.Condition(loop=self.loop),
            asyncio.Semaphore(loop=self.loop),
            asyncio.BoundedSemaphore(loop=self.loop),
        ]

        async def test(lock):
            await asyncio.sleep(0.01, loop=self.loop)
            self.assertFalse(lock.locked())
            async with lock as _lock:
                self.assertIs(_lock, None)
                self.assertTrue(lock.locked())
                await asyncio.sleep(0.01, loop=self.loop)
                self.assertTrue(lock.locked())
            self.assertFalse(lock.locked())

        for primitive in primitives:
            self.loop.run_until_complete(test(primitive))
            self.assertFalse(primitive.locked())


class StreamReaderTests(BaseTest):
    def test_readline(self):
        DATA = b'line1\nline2\nline3'

        stream = asyncio.StreamReader(loop=self.loop)
        stream.feed_data(DATA)
        stream.feed_eof()

        async def reader():
            data = []
            async for line in stream:
                data.append(line)
            return data

        data = self.loop.run_until_complete(reader())
        self.assertEqual(data, [b'line1\n', b'line2\n', b'line3'])


class CoroutineTests(BaseTest):
    def test_iscoroutine(self):
        async def foo():
            pass

        f = foo()
        try:
            self.assertTrue(asyncio.iscoroutine(f))
        finally:
            f.close()  # silence warning

        # Test that asyncio.iscoroutine() uses collections.abc.Coroutine
        class FakeCoro:
            def send(self, value):
                pass

            def throw(self, typ, val=None, tb=None):
                pass

            def close(self):
                pass

            def __await__(self):
                yield

        self.assertTrue(asyncio.iscoroutine(FakeCoro()))

    def test_iscoroutinefunction(self):
        async def foo():
            pass

        self.assertTrue(asyncio.iscoroutinefunction(foo))

    def test_function_returning_awaitable(self):
        class Awaitable:
            def __await__(self):
                return ('spam',)

        @asyncio.coroutine
        def func():
            return Awaitable()

        coro = func()
        self.assertEqual(coro.send(None), 'spam')
        coro.close()

    def test_async_def_coroutines(self):
        async def bar():
            return 'spam'

        async def foo():
            return await bar()

        # production mode
        data = self.loop.run_until_complete(foo())
        self.assertEqual(data, 'spam')

        # debug mode
        self.loop.set_debug(True)
        data = self.loop.run_until_complete(foo())
        self.assertEqual(data, 'spam')

    @pytest.mark.xfail(msg="XXX doesn't always trigger")
    @mock.patch('asyncio.coroutines.logger')
    def test_async_def_wrapped(self, m_log):
        async def foo():
            pass

        async def start():
            foo_coro = foo()
            self.assertRegex(repr(foo_coro), r'<CoroWrapper .*\.foo\(\) running at .*pep492.*>')

            with support.check_warnings((r'.*foo.*was never', RuntimeWarning)):
                foo_coro = None
                support.gc_collect()
                self.assertTrue(m_log.error.called)
                message = m_log.error.call_args[0][0]
                self.assertRegex(message, r'CoroWrapper.*foo.*was never')

        self.loop.set_debug(True)
        self.loop.run_until_complete(start())

        async def start():
            foo_coro = foo()
            task = asyncio.ensure_future(foo_coro, loop=self.loop)
            self.assertRegex(repr(task), r'Task.*foo.*running')

        self.loop.run_until_complete(start())

    def test_types_coroutine(self):
        def gen():
            yield from ()
            return 'spam'

        @types.coroutine
        def func():
            return gen()

        async def coro():
            wrapper = func()
            self.assertIsInstance(wrapper, types._GeneratorWrapper)
            return await wrapper

        data = self.loop.run_until_complete(coro())
        self.assertEqual(data, 'spam')

    def test_task_print_stack(self):
        T = None

        async def foo():
            f = T.get_stack(limit=1)
            try:
                self.assertEqual(f[0].f_code.co_name, 'foo')
            finally:
                f = None

        async def runner():
            nonlocal T
            T = asyncio.ensure_future(foo(), loop=self.loop)
            await T

        self.loop.run_until_complete(runner())

    def test_double_await(self):
        async def afunc():
            await asyncio.sleep(0.1, loop=self.loop)

        async def runner():
            coro = afunc()
            t = asyncio.Task(coro, loop=self.loop)
            try:
                await asyncio.sleep(0, loop=self.loop)
                await coro
            finally:
                t.cancel()

        self.loop.set_debug(True)
        with self.assertRaisesRegex(RuntimeError,
                                    r'Cannot await.*test_double_await.*\bafunc\b.*while.*\bsleep\b'
                                    ) if sys.version_info < (3, 7) else self.assertRaises(
                                        RuntimeError, msg='coroutine is being awaited already'):

            self.loop.run_until_complete(runner())


if __name__ == '__main__':
    unittest.main()



# asyncio.loop logs some things we might need to see.
import logging
logging.basicConfig()
del logging

# asyncio runs many error messages through reprlib,
# which defaults to fairly short strings,
# which is a major PITA.
from reprlib import aRepr
aRepr.maxstring = 9999
aRepr.maxother = 9999
aRepr.maxlong = 9999
del aRepr

# Find the test SSL keys.
from . import utils as test_utils


def finish_request(self, request, client_address):
    import os
    import ssl
    here = os.path.join(os.path.dirname(__file__), 'python')
    keyfile = os.path.join(here, 'ssl_key.pem')
    certfile = os.path.join(here, 'ssl_cert.pem')
    context = ssl.SSLContext()
    context.load_cert_chain(certfile, keyfile)

    ssock = context.wrap_socket(request, server_side=True)
    try:
        self.RequestHandlerClass(ssock, client_address, self)
        ssock.close()
    except OSError:
        # maybe socket has been closed by peer
        pass


test_utils.SSLWSGIServerMixin.finish_request = finish_request
del test_utils
del finish_request



# This code implements a clone of the asyncio mainloop which hooks into
# Trio.

import sys
import trio
import asyncio
import warnings
import threading
from contextvars import ContextVar

from .util import run_future
from .async_ import TrioEventLoop, open_loop

try:
    from trio.hazmat import wait_for_child
except ImportError:
    from .child import wait_for_child

import logging
logger = logging.getLogger(__name__)

__all__ = [
    'run',
    'run_trio_task',
    'run_trio',
    'run_future',
    'run_coroutine',
    'run_asyncio',
    'wrap_generator',
    'TrioChildWatcher',
    'TrioPolicy',
]

current_loop = ContextVar('trio_aio_loop', default=None)
current_policy = ContextVar('trio_aio_policy', default=None)


class _TrioPolicy(asyncio.events.BaseDefaultEventLoopPolicy):
    _loop_factory = TrioEventLoop

    def new_event_loop(self):
        try:
            trio.hazmat.current_task()
        except RuntimeError:
            if 'pytest' not in sys.modules:
                warnings.warn(
                    "trio_asyncio should be used from within a Trio event loop.",
                    DeprecationWarning,
                    stacklevel=2
                )
            from .sync import SyncTrioEventLoop
            loop = SyncTrioEventLoop()
            return loop
        else:
            raise RuntimeError(
                "You're within a Trio environment.\n"
                "Use 'async with open_loop()' instead."
            )

    def get_event_loop(self):
        """Get the current event loop.

        Note that this will auto-generate an event loop if none exists, for
        compatibility with asyncio.

        To get a Trio-compatible asyncio loop, use
        ``async with trio_asyncio.open_loop() as loop:``.

        To test whether an event loop is running, check the loop policy's
        ``.current_event_loop`` property.
        """
        try:
            trio.hazmat.current_task()
        except RuntimeError:  # no Trio task is active
            # this creates a new loop in the main task
            return super().get_event_loop()
        else:
            return current_loop.get()

    @property
    def current_event_loop(self):
        """The currently-running event loop, if one exists."""
        loop = current_loop.get()
        if loop is None:
            loop = super().get_event_loop()
        return loop

    def set_event_loop(self, loop):
        """Set the current event loop."""
        current_loop.set(loop)


# We need to monkey-patch asyncio's policy+loop getters to return our
# TrioPolicy and the current loop whenever we are within Trio.

from asyncio import events as _aio_event

#####

_orig_policy_get = _aio_event.get_event_loop_policy


def _new_policy_get():
    try:
        task = trio.hazmat.current_task()
    except RuntimeError:
        policy = None
    else:
        policy = task.context.get(current_policy, None)
    if policy is None:
        policy = _orig_policy_get()
    return policy


_aio_event.get_event_loop_policy = _new_policy_get
asyncio.get_event_loop_policy = _new_policy_get

#####

_orig_run_get = _aio_event._get_running_loop


def _new_run_get():
    try:
        task = trio.hazmat.current_task()
    except RuntimeError:
        loop = None
    else:
        loop = task.context.get(current_loop, None)

    if loop is None:
        loop = _orig_run_get()
    return loop


_aio_event._get_running_loop = _new_run_get

#####

_orig_loop_get = _aio_event.get_event_loop


def _new_loop_get():
    loop = _new_run_get()
    if loop is None:
        loop = _orig_loop_get()
    return loop


_aio_event.get_event_loop = _new_loop_get
asyncio.get_event_loop = _new_loop_get


class TrioPolicy(_TrioPolicy, asyncio.DefaultEventLoopPolicy):
    """This is the loop policy that's active whenever we're in a Trio context."""

    def _init_watcher(self):
        with asyncio.events._lock:
            if self._watcher is None:  # pragma: no branch
                self._watcher = TrioChildWatcher()
                if isinstance(threading.current_thread(), threading._MainThread):
                    self._watcher.attach_loop(current_loop.get())

        if self._watcher is not None and \
                isinstance(threading.current_thread(), threading._MainThread):
            self._watcher.attach_loop(current_loop.get())

    def set_child_watcher(self, watcher):
        if watcher is not None:
            if not isinstance(watcher, TrioChildWatcher):
                # raise RuntimeError("You must use a TrioChildWatcher here. "
                #                    "Sorry.")
                # warnings.warn("You must use a TrioChildWatcher.")
                #
                loop = watcher._loop  # ugh.
                watcher.close()
                watcher = TrioChildWatcher()
                watcher.attach_loop(loop)
        super().set_child_watcher(watcher)


class TrioChildWatcher(asyncio.AbstractChildWatcher if sys.platform != 'win32' else object):
    # AbstractChildWatcher not available under Windows
    def __init__(self):
        super().__init__()
        self._callbacks = {}  # pid => handler

    def attach_loop(self, loop):
        self._loop = loop

    async def _waitpid(self, pid, callback, *args):
        returncode = await wait_for_child(pid)
        callback(pid, returncode, *args)

    def add_child_handler(self, pid, callback, *args):
        """Add a callback to run when a child process terminates."""
        h = self._loop.run_trio(self._waitpid, pid, callback, *args)
        self._callbacks[pid] = h

    def remove_child_handler(self, pid):
        """Remove the callback to run when a child process terminates."""
        h = self._callbacks.pop(pid, None)
        if h is None:
            return False
        h.cancel()
        return True

    def close(self):
        for pid in list(self._callbacks):
            h = self._callbacks.pop(pid, None)
            if h is None:
                continue
            h.cancel()
        self._loop = None

    def __enter__(self):
        return self

    def __exit__(self, *tb):
        self.close()


def wrap_generator(proc, *args):
    loop = asyncio.get_event_loop()
    if not isinstance(loop, TrioEventLoop):
        raise RuntimeError("Need to run in a trio_asyncio.open_loop() context")
    return loop.wrap_generator(proc, *args)


async def run_asyncio(proc, *args):
    """Run an asyncio function or method from Trio.

    :return: whatever the procedure returns.
    :raises: whatever the procedure raises.

    This is a Trio coroutine.
    """

    loop = asyncio.get_event_loop()
    if not isinstance(loop, TrioEventLoop):
        raise RuntimeError("Need to run in a trio_asyncio.open_loop() context")
    return await loop.run_asyncio(proc, *args)


async def run_coroutine(fut):
    """Wait for an asyncio future/coroutine.

    Cancelling the current Trio scope will cancel the future/coroutine.

    Cancelling the future/coroutine will cause an
    ``asyncio.CancelledError``.

    This is a Trio coroutine.
    """

    loop = asyncio.get_event_loop()
    if not isinstance(loop, TrioEventLoop):
        raise RuntimeError("Need to run in a trio_asyncio.open_loop() context")
    return await loop.run_coroutine(fut)


def run_trio(proc, *args):
    """Call an asynchronous Trio function from asyncio.

    Returns a Future with the result / exception.

    Cancelling the future will cancel the Trio task running your
    function, or prevent it from starting if that is still possible.

    You need to handle errors yourself.
    """
    loop = asyncio.get_event_loop()
    if not isinstance(loop, TrioEventLoop):  # pragma: no cover
        raise RuntimeError("Need to run in a trio_asyncio.open_loop() context")
    return loop.run_trio(proc, *args)


def run_trio_task(proc, *args):
    """Call an asynchronous Trio function from sync context.

    This method queues the task and returns immediately.
    It does not return a value.

    An uncaught error will propagate to, and terminate, the trio-asyncio loop.
    """
    loop = asyncio.get_event_loop()
    if not isinstance(loop, TrioEventLoop):
        raise RuntimeError("Need to run in a trio_asyncio.open_loop() context")
    loop.run_trio_task(proc, *args)


def run(proc, *args, queue_len=None):
    """Like :func:`trio.run`, but adds a context that supports asyncio.
    """

    async def _run_task(proc, args):
        async with open_loop(queue_len=queue_len):
            return await proc(*args)

    trio.run(_run_task, proc, args)



from tests import aiotest
import pytest
import trio
import trio_asyncio


class TestThread(aiotest.TestCase):
    @pytest.mark.trio
    async def test_ident(self, loop, config):
        threading = config.threading
        try:
            get_ident = threading.get_ident  # Python 3
        except AttributeError:
            get_ident = threading._get_ident  # Python 2

        result = {'ident': None}

        def work():
            result['ident'] = get_ident()

        fut = loop.run_in_executor(None, work)
        await loop.run_coroutine(fut)

        # ensure that work() was executed in a different thread
        work_ident = result['ident']
        assert work_ident is not None
        assert work_ident != get_ident()

    @pytest.mark.trio
    async def test_run_twice(self, loop):
        result = []

        def work():
            result.append("run")

        fut = loop.run_in_executor(None, work)
        await loop.run_future(fut)
        assert result == ["run"]

        # ensure that run_in_executor() can be called twice
        fut = loop.run_in_executor(None, work)
        await loop.run_future(fut)
        assert result == ["run", "run"]

    @pytest.mark.trio
    async def test_policy(self, loop, config):
        asyncio = config.asyncio
        result = {'loop': 'not set'}  # sentinel, different than None

        def work():
            try:
                result['loop'] = asyncio.get_event_loop()
            except Exception as exc:
                result['loop'] = exc

        # get_event_loop() must return None in a different thread
        fut = loop.run_in_executor(None, work)
        await loop.run_future(fut)
        assert isinstance(result['loop'], (AssertionError, RuntimeError))

    @pytest.mark.trio
    async def test_run_in_thread(self, config):
        threading = config.threading

        class LoopThread(threading.Thread):
            def __init__(self, event):
                super(LoopThread, self).__init__()
                self.loop = None
                self.event = event

            async def _run(self):
                async with trio_asyncio.open_loop() as loop:
                    self.loop = loop
                    loop.set_debug(True)

                    self.event.set()
                    await loop.wait_stopped()

            def run(self):
                trio.run(self._run)

        result = []

        # start an event loop in a thread
        event = threading.Event()
        thread = LoopThread(event)
        thread.start()
        event.wait()
        loop = thread.loop

        def func(loop):
            result.append(threading.current_thread().ident)
            loop.stop()

        # call func() in a different thread using the event loop
        tid = thread.ident
        loop.call_soon_threadsafe(func, loop)

        # wait for the other thread's event loop to terminate
        thread.join()
        assert result == [tid]



# XX this does not belong here -- b/c it's here, these things only apply to
# the tests in trio/_core/tests, not in trio/tests. For now there's some
# copy-paste...
#
# this stuff should become a proper pytest plugin

import pytest
import asyncio
import trio_asyncio
import inspect

# Hacks for <3.7
if not hasattr(asyncio, 'run'):

    def run(main, *, debug=False):
        loop = asyncio.new_event_loop()
        loop.set_debug(debug)
        return loop.run(main)

    asyncio.run = run

if not hasattr(asyncio, 'current_task'):

    def current_task(loop=None):
        return asyncio.Task.current_task(loop)

    asyncio.current_task = current_task

if not hasattr(asyncio, 'all_tasks'):

    def all_tasks(loop=None):
        return asyncio.Task.all_tasks(loop)

    asyncio.all_tasks = all_tasks

if not hasattr(asyncio, 'create_task'):

    if hasattr(asyncio.events, 'get_running_loop'):

        def create_task(coro):
            loop = asyncio.events.get_running_loop()
            return loop.create_task(coro)
    else:

        def create_task(coro):
            loop = asyncio.events._get_running_loop()
            return loop.create_task(coro)

    asyncio.create_task = create_task


@pytest.fixture
async def loop():
    async with trio_asyncio.open_loop() as loop:
        try:
            yield loop
        finally:
            await loop.stop().wait()


@pytest.fixture
def sync_loop():
    loop = asyncio.new_event_loop()
    with loop:
        yield loop


# auto-trio-ize all async functions
@pytest.hookimpl(tryfirst=True)
def pytest_pyfunc_call(pyfuncitem):
    if inspect.iscoroutinefunction(pyfuncitem.obj):
        pyfuncitem.obj = pytest.mark.trio(pyfuncitem.obj)


_old_policy = asyncio.get_event_loop_policy()
_new_policy = trio_asyncio.TrioPolicy()
asyncio.set_event_loop_policy(_new_policy)


@pytest.fixture
def old_policy():
    asyncio.set_event_loop_policy(_old_policy)
    try:
        yield _old_policy
    finally:
        asyncio.set_event_loop_policy(_new_policy)



# "High-level" networking interface

import os
try:
    import fcntl
except ImportError:
    _fcntl = False
else:
    _fcntl = True
import errno

import trio
from trio._util import ConflictDetector
from trio.abc import SendStream, ReceiveStream, AsyncResource

__all__ = ["ReadFDStream", "WriteFDStream"]

_closed_stream_errnos = {
    # Unix
    errno.EBADF,
    # Windows
    errno.ENOTSOCK,
}


class _FDStream(AsyncResource):
    def __init__(self, fd):
        if hasattr(fd, 'fileno'):
            # Unwrap Python's IO buffers.
            while hasattr(fd, 'detach'):
                fd = fd.detach()
            # _io.FileIO.closefd is not writeable and we don't want to hold
            # a reference to the original file descriptor, so take the easy
            # way out
            fd = os.dup(fd.fileno())
        if not isinstance(fd, int) or fd < 0:
            raise TypeError("ReadFDStream requires a file descriptor")

        self._fd = fd
        if _fcntl:
            self._fdflags = fcntl.fcntl(fd, fcntl.F_GETFL, 0)
            fcntl.fcntl(fd, fcntl.F_SETFL, self._fdflags | os.O_NONBLOCK)

        self._send_conflict_detector = ConflictDetector(
            "another task is currently sending data to this ReadFDStream"
        )

    def fileno(self):
        return self._fd

    async def aclose(self):
        self.close()
        await trio.hazmat.checkpoint()

    def close(self):
        try:
            # fcntl.fcntl(self._fd, fcntl.F_SETFL, self._fdflags)
            os.close(self._fd)
        except OSError as err:
            if err.errno != errno.EBADF:
                raise


class ReadFDStream(_FDStream, ReceiveStream):
    """An implementation of the :class:`trio.abc.ReceiveStream`
    interface based on a raw file descriptor.

    Args:
      fd: The file descriptor to wrap.

    The file descriptor will *not* be closed by this object's :meth:`close`
    or :meth:`aclose` methods.
    """

    async def receive_some(self, max_bytes):
        if max_bytes < 1:
            await trio.hazmat.checkpoint()
            raise ValueError("max_bytes must be >= 1")
        await trio.hazmat.wait_readable(self._fd)
        return os.read(self._fd, max_bytes)


class WriteFDStream(_FDStream, SendStream):
    """An implementation of the :class:`trio.abc.SendStream`
    interface based on a raw file descriptor.

    Args:
      fd: The file descriptor to wrap.

    The file descriptor will *not* be closed by this object's :meth:`close`
    or :meth:`aclose` methods.
    """

    async def send_all(self, data):
        with self._send_conflict_detector.sync:
            with memoryview(data) as data:
                if not data:
                    await trio.hazmat.checkpoint()
                    return
                total_sent = 0
                while total_sent < len(data):
                    with data[total_sent:] as remaining:
                        await trio.hazmat.wait_writable(self._fd)
                        sent = os.write(self._fd, remaining)
                    total_sent += sent

    async def wait_send_all_might_not_block(self):
        async with self._send_conflict_detector:
            await self.socket.wait_writable()



import trio
import trio_asyncio
import asyncio
import pytest

# Tests for concurrent or nested loops


@pytest.mark.trio
async def test_parallel():
    loops = [None, None]
    async with trio.open_nursery() as n:

        async def gen_loop(i, task_status=trio.TASK_STATUS_IGNORED):
            task_status.started()
            async with trio_asyncio.open_loop() as loop:
                loops[i] = loop

        assert not isinstance(asyncio._get_running_loop(), trio_asyncio.TrioEventLoop)
        await n.start(gen_loop, 0)
        await n.start(gen_loop, 1)

    assert isinstance(loops[0], trio_asyncio.TrioEventLoop)
    assert isinstance(loops[1], trio_asyncio.TrioEventLoop)
    assert loops[0] is not loops[1]


@pytest.mark.trio
async def test_nested():
    loops = [None, None]
    async with trio.open_nursery() as n:

        async def gen_loop(i, task_status=trio.TASK_STATUS_IGNORED):
            task_status.started()
            async with trio_asyncio.open_loop() as loop:
                loops[i] = loop
                if i > 0:
                    await n.start(gen_loop, i - 1)

        assert not isinstance(asyncio._get_running_loop(), trio_asyncio.TrioEventLoop)
        await n.start(gen_loop, 1)
    assert not isinstance(asyncio._get_running_loop(), trio_asyncio.TrioEventLoop)
    assert isinstance(loops[0], trio_asyncio.TrioEventLoop)
    assert isinstance(loops[1], trio_asyncio.TrioEventLoop)
    assert loops[0] is not loops[1]


async def _test_same_task():
    assert not isinstance(asyncio.get_event_loop_policy(), trio_asyncio.TrioPolicy)

    def get_loop(i, loop, policy):
        assert loop == asyncio.get_event_loop()
        assert policy == asyncio.get_event_loop_policy()

    async with trio.open_nursery():
        async with trio_asyncio.open_loop() as loop1:
            policy = asyncio.get_event_loop_policy()
            assert isinstance(policy, trio_asyncio.TrioPolicy)
            async with trio_asyncio.open_loop() as loop2:
                assert policy == asyncio.get_event_loop_policy()
                loop1.call_later(0.1, get_loop, 0, loop1, policy)
                loop2.call_later(0.1, get_loop, 1, loop2, policy)
                await trio.sleep(0.2)

    assert not isinstance(asyncio.get_event_loop_policy(), trio_asyncio.TrioPolicy)
    assert not isinstance(asyncio._get_running_loop(), trio_asyncio.TrioEventLoop)


def test_same_task(old_policy):
    assert not isinstance(asyncio.get_event_loop_policy(), trio_asyncio.TrioPolicy)
    trio.run(_test_same_task)



from __future__ import absolute_import
from tests.aiotest import socketpair
import pytest
import trio


class TestAddReader:
    @pytest.mark.trio
    async def test_add_reader(self, loop):
        result = {'received': None}
        rsock, wsock = socketpair()
        ready = trio.Event()
        try:

            def reader():
                data = rsock.recv(100)
                result['received'] = data
                loop.remove_reader(rsock)
                ready.set()

            def writer():
                loop.remove_writer(wsock)
                loop.call_soon(wsock.send, b'abc')

            loop.add_reader(rsock, reader)
            loop.add_writer(wsock, writer)

            await ready.wait()
            assert result['received'] == b'abc'

        finally:
            rsock.close()
            wsock.close()

    async def check_add_replace(self, event, loop, config):
        socket = config.socket

        selector = loop._selector
        if event == 'reader':
            add_sock = loop.add_reader
            remove_sock = loop.remove_reader

            def get_handle(fileobj):
                return selector.get_key(fileobj).data[0]
        else:
            add_sock = loop.add_writer
            remove_sock = loop.remove_writer

            def get_handle(fileobj):
                return selector.get_key(fileobj).data[1]

        sock = socket.socket()
        try:

            def func():
                pass

            def func2():
                pass

            with pytest.raises(KeyError):
                get_handle(sock)

            add_sock(sock, func)
            handle1 = get_handle(sock)
            assert not handle1._cancelled

            add_sock(sock, func2)
            handle2 = get_handle(sock)
            assert handle1 is not handle2
            assert handle1._cancelled
            assert not handle2._cancelled

            removed = remove_sock(sock)
            assert removed
            assert handle2._cancelled

            removed = remove_sock(sock)
            assert not removed
        finally:
            sock.close()

    @pytest.mark.trio
    async def test_add_reader_replace(self, loop, config):
        await self.check_add_replace("reader", loop, config)

    @pytest.mark.trio
    async def test_add_writer_replace(self, loop, config):
        await self.check_add_replace("writer", loop, config)



import pytest
import trio_asyncio
import asyncio
import trio

# from tests import aiotest


class Seen:
    flag = 0


class TestMisc:
    @pytest.mark.trio
    async def test_close_no_stop(self):
        with pytest.raises(RuntimeError):
            async with trio_asyncio.open_loop() as loop:

                def close_no_stop():
                    loop.close()

                loop.call_soon(close_no_stop)

                await trio.sleep(0.1)
                await loop.wait_closed()

    @pytest.mark.trio
    async def test_err1(self, loop):
        async def raise_err():
            raise RuntimeError("Foo")

        with pytest.raises(RuntimeError) as err:
            await loop.run_asyncio(raise_err)
        assert err.value.args[0] == "Foo"

    @pytest.mark.trio
    async def test_err3(self, loop):
        owch = 0

        async def nest():
            nonlocal owch
            owch = 1
            raise RuntimeError("Hello")

        async def call_nested():
            with pytest.raises(RuntimeError) as err:
                await loop.run_trio(nest)
            assert err.value.args[0] == "Hello"

        await loop.run_asyncio(call_nested)
        assert owch

    @pytest.mark.trio
    async def test_run(self, loop):
        owch = 0

        async def nest():
            await trio.sleep(0.01)
            nonlocal owch
            owch = 1

        async def call_nested():
            await loop.run_trio(nest)

        await loop.run_asyncio(call_nested)
        assert owch

    async def _test_run(self):
        owch = 0

        async def nest():
            await trio.sleep(0.01)
            nonlocal owch
            owch = 1

        async def call_nested():
            await trio_asyncio.run_trio(nest)

        await trio_asyncio.run_asyncio(call_nested)
        assert owch

    def test_run2(self):
        trio_asyncio.run(self._test_run)

    @pytest.mark.trio
    async def test_run_task(self):
        owch = 0

        async def nest(x):
            nonlocal owch
            owch += x

        with pytest.raises(RuntimeError):
            trio_asyncio.run_trio_task(nest, 100)

        with pytest.raises(RuntimeError):
            with trio_asyncio.open_loop():
                nest(1000)

        async with trio_asyncio.open_loop():
            trio_asyncio.run_trio_task(nest, 1)
        await trio.sleep(0.05)
        assert owch == 1

    @pytest.mark.trio
    async def test_err2(self, loop):
        owch = 0

        async def nest():
            nonlocal owch
            owch = 1
            raise RuntimeError("Hello")

        async def call_nested():
            await loop.run_asyncio(nest)

        async def call_more_nested():
            with pytest.raises(RuntimeError) as err:
                await loop.run_trio(call_nested)
            assert err.value.args[0] == "Hello"

        await loop.run_asyncio(call_more_nested)
        assert owch

    @pytest.mark.trio
    async def test_run3(self, loop):
        owch = 0

        async def nest():
            nonlocal owch
            owch = 1

        async def call_nested():
            await loop.run_asyncio(nest)

        async def call_more_nested():
            await loop.run_trio(call_nested)

        await loop.run_asyncio(call_more_nested)
        assert owch

    @pytest.mark.trio
    async def test_cancel_sleep(self, loop):
        owch = 0

        def do_not_run():
            nonlocal owch
            owch = 1
            raise Exception("should not run")

        async def cancel_sleep():
            h = loop.call_later(0.2, do_not_run)
            await asyncio.sleep(0.1, loop=loop)
            h.cancel()
            await asyncio.sleep(0.3, loop=loop)

        await loop.run_asyncio(cancel_sleep)
        assert owch == 0



"""Tests for proactor_events.py"""

import sys
import socket
import unittest
from unittest import mock

import asyncio
from asyncio.proactor_events import BaseProactorEventLoop
from asyncio.proactor_events import _ProactorSocketTransport
from asyncio.proactor_events import _ProactorWritePipeTransport
from asyncio.proactor_events import _ProactorDuplexPipeTransport
from .. import utils as test_utils


def close_transport(transport):
    # Don't call transport.close() because the event loop and the IOCP proactor
    # are mocked
    if transport._sock is None:
        return
    transport._sock.close()
    transport._sock = None


class ProactorSocketTransportTests(test_utils.TestCase):
    def setUp(self):
        super().setUp()
        self.loop = self.new_test_loop()
        self.addCleanup(self.loop.close)
        self.proactor = mock.Mock()
        self.loop._proactor = self.proactor
        self.protocol = test_utils.make_test_protocol(asyncio.Protocol)
        self.sock = mock.Mock(socket.socket)

    def socket_transport(self, waiter=None):
        transport = _ProactorSocketTransport(self.loop, self.sock, self.protocol, waiter=waiter)
        self.addCleanup(close_transport, transport)
        return transport

    def test_ctor(self):
        fut = asyncio.Future(loop=self.loop)
        tr = self.socket_transport(waiter=fut)
        test_utils.run_briefly(self.loop)
        self.assertIsNone(fut.result())
        self.protocol.connection_made(tr)
        try:
            self.proactor.recv.assert_called_with(self.sock, 4096)
        except AssertionError:
            self.proactor.recv.assert_called_with(self.sock, 32768)

    def test_loop_reading(self):
        tr = self.socket_transport()
        tr._loop_reading()
        try:
            self.loop._proactor.recv.assert_called_with(self.sock, 4096)
        except AssertionError:
            self.loop._proactor.recv.assert_called_with(self.sock, 32768)
        self.assertFalse(self.protocol.data_received.called)
        self.assertFalse(self.protocol.eof_received.called)

    def test_loop_reading_data(self):
        res = asyncio.Future(loop=self.loop)
        res.set_result(b'data')

        tr = self.socket_transport()
        tr._read_fut = res
        tr._loop_reading(res)
        try:
            self.loop._proactor.recv.assert_called_with(self.sock, 4096)
        except AssertionError:
            self.loop._proactor.recv.assert_called_with(self.sock, 32768)
        self.protocol.data_received.assert_called_with(b'data')

    def test_loop_reading_no_data(self):
        res = asyncio.Future(loop=self.loop)
        res.set_result(b'')

        tr = self.socket_transport()
        self.assertRaises(AssertionError, tr._loop_reading, res)

        tr.close = mock.Mock()
        tr._read_fut = res
        tr._loop_reading(res)
        self.assertFalse(self.loop._proactor.recv.called)
        self.assertTrue(self.protocol.eof_received.called)
        self.assertTrue(tr.close.called)

    def test_loop_reading_aborted(self):
        err = self.loop._proactor.recv.side_effect = ConnectionAbortedError()

        tr = self.socket_transport()
        tr._fatal_error = mock.Mock()
        tr._loop_reading()
        tr._fatal_error.assert_called_with(err, 'Fatal read error on pipe transport')

    def test_loop_reading_aborted_closing(self):
        self.loop._proactor.recv.side_effect = ConnectionAbortedError()

        tr = self.socket_transport()
        tr._closing = True
        tr._fatal_error = mock.Mock()
        tr._loop_reading()
        self.assertFalse(tr._fatal_error.called)

    def test_loop_reading_aborted_is_fatal(self):
        self.loop._proactor.recv.side_effect = ConnectionAbortedError()
        tr = self.socket_transport()
        tr._closing = False
        tr._fatal_error = mock.Mock()
        tr._loop_reading()
        self.assertTrue(tr._fatal_error.called)

    def test_loop_reading_conn_reset_lost(self):
        err = self.loop._proactor.recv.side_effect = ConnectionResetError()

        tr = self.socket_transport()
        tr._closing = False
        tr._fatal_error = mock.Mock()
        tr._force_close = mock.Mock()
        tr._loop_reading()
        self.assertFalse(tr._fatal_error.called)
        tr._force_close.assert_called_with(err)

    def test_loop_reading_exception(self):
        err = self.loop._proactor.recv.side_effect = (OSError())

        tr = self.socket_transport()
        tr._fatal_error = mock.Mock()
        tr._loop_reading()
        tr._fatal_error.assert_called_with(err, 'Fatal read error on pipe transport')

    def test_write(self):
        tr = self.socket_transport()
        tr._loop_writing = mock.Mock()
        tr.write(b'data')
        self.assertEqual(tr._buffer, None)
        tr._loop_writing.assert_called_with(data=b'data')

    def test_write_no_data(self):
        tr = self.socket_transport()
        tr.write(b'')
        self.assertFalse(tr._buffer)

    def test_write_more(self):
        tr = self.socket_transport()
        tr._write_fut = mock.Mock()
        tr._loop_writing = mock.Mock()
        tr.write(b'data')
        self.assertEqual(tr._buffer, b'data')
        self.assertFalse(tr._loop_writing.called)

    def test_loop_writing(self):
        tr = self.socket_transport()
        tr._buffer = bytearray(b'data')
        tr._loop_writing()
        self.loop._proactor.send.assert_called_with(self.sock, b'data')
        self.loop._proactor.send.return_value.add_done_callback.\
            assert_called_with(tr._loop_writing)

    @mock.patch('asyncio.proactor_events.logger')
    def test_loop_writing_err(self, m_log):
        err = self.loop._proactor.send.side_effect = OSError()
        tr = self.socket_transport()
        tr._fatal_error = mock.Mock()
        tr._buffer = [b'da', b'ta']
        tr._loop_writing()
        tr._fatal_error.assert_called_with(err, 'Fatal write error on pipe transport')
        tr._conn_lost = 1

        tr.write(b'data')
        tr.write(b'data')
        tr.write(b'data')
        tr.write(b'data')
        tr.write(b'data')
        self.assertEqual(tr._buffer, None)
        m_log.warning.assert_called_with('socket.send() raised exception.')

    def test_loop_writing_stop(self):
        fut = asyncio.Future(loop=self.loop)
        fut.set_result(b'data')

        tr = self.socket_transport()
        tr._write_fut = fut
        tr._loop_writing(fut)
        self.assertIsNone(tr._write_fut)

    def test_loop_writing_closing(self):
        fut = asyncio.Future(loop=self.loop)
        fut.set_result(1)

        tr = self.socket_transport()
        tr._write_fut = fut
        tr.close()
        tr._loop_writing(fut)
        self.assertIsNone(tr._write_fut)
        test_utils.run_briefly(self.loop)
        self.protocol.connection_lost.assert_called_with(None)

    def test_abort(self):
        tr = self.socket_transport()
        tr._force_close = mock.Mock()
        tr.abort()
        tr._force_close.assert_called_with(None)

    def test_close(self):
        tr = self.socket_transport()
        tr.close()
        test_utils.run_briefly(self.loop)
        self.protocol.connection_lost.assert_called_with(None)
        self.assertTrue(tr.is_closing())
        self.assertEqual(tr._conn_lost, 1)

        self.protocol.connection_lost.reset_mock()
        tr.close()
        test_utils.run_briefly(self.loop)
        self.assertFalse(self.protocol.connection_lost.called)

    def test_close_write_fut(self):
        tr = self.socket_transport()
        tr._write_fut = mock.Mock()
        tr.close()
        test_utils.run_briefly(self.loop)
        self.assertFalse(self.protocol.connection_lost.called)

    def test_close_buffer(self):
        tr = self.socket_transport()
        tr._buffer = [b'data']
        tr.close()
        test_utils.run_briefly(self.loop)
        self.assertFalse(self.protocol.connection_lost.called)

    @mock.patch('asyncio.base_events.logger')
    def test_fatal_error(self, m_logging):
        tr = self.socket_transport()
        tr._force_close = mock.Mock()
        tr._fatal_error(None)
        self.assertTrue(tr._force_close.called)
        self.assertTrue(m_logging.error.called)

    def test_force_close(self):
        tr = self.socket_transport()
        tr._buffer = [b'data']
        read_fut = tr._read_fut = mock.Mock()
        write_fut = tr._write_fut = mock.Mock()
        tr._force_close(None)

        read_fut.cancel.assert_called_with()
        write_fut.cancel.assert_called_with()
        test_utils.run_briefly(self.loop)
        self.protocol.connection_lost.assert_called_with(None)
        self.assertEqual(None, tr._buffer)
        self.assertEqual(tr._conn_lost, 1)

    def test_force_close_idempotent(self):
        tr = self.socket_transport()
        tr._closing = True
        tr._force_close(None)
        test_utils.run_briefly(self.loop)
        self.assertFalse(self.protocol.connection_lost.called)

    def test_fatal_error_2(self):
        tr = self.socket_transport()
        tr._buffer = [b'data']
        tr._force_close(None)

        test_utils.run_briefly(self.loop)
        self.protocol.connection_lost.assert_called_with(None)
        self.assertEqual(None, tr._buffer)

    def test_call_connection_lost(self):
        tr = self.socket_transport()
        tr._call_connection_lost(None)
        self.assertTrue(self.protocol.connection_lost.called)
        self.assertTrue(self.sock.close.called)

    def test_write_eof(self):
        tr = self.socket_transport()
        self.assertTrue(tr.can_write_eof())
        tr.write_eof()
        self.sock.shutdown.assert_called_with(socket.SHUT_WR)
        tr.write_eof()
        self.assertEqual(self.sock.shutdown.call_count, 1)
        tr.close()

    def test_write_eof_buffer(self):
        tr = self.socket_transport()
        f = asyncio.Future(loop=self.loop)
        tr._loop._proactor.send.return_value = f
        tr.write(b'data')
        tr.write_eof()
        self.assertTrue(tr._eof_written)
        self.assertFalse(self.sock.shutdown.called)
        tr._loop._proactor.send.assert_called_with(self.sock, b'data')
        f.set_result(4)
        self.loop._run_once()
        self.sock.shutdown.assert_called_with(socket.SHUT_WR)
        tr.close()

    def test_write_eof_write_pipe(self):
        tr = _ProactorWritePipeTransport(self.loop, self.sock, self.protocol)
        self.assertTrue(tr.can_write_eof())
        tr.write_eof()
        self.assertTrue(tr.is_closing())
        self.loop._run_once()
        self.assertTrue(self.sock.close.called)
        tr.close()

    def test_write_eof_buffer_write_pipe(self):
        tr = _ProactorWritePipeTransport(self.loop, self.sock, self.protocol)
        f = asyncio.Future(loop=self.loop)
        tr._loop._proactor.send.return_value = f
        tr.write(b'data')
        tr.write_eof()
        self.assertTrue(tr.is_closing())
        self.assertFalse(self.sock.shutdown.called)
        tr._loop._proactor.send.assert_called_with(self.sock, b'data')
        f.set_result(4)
        self.loop._run_once()
        self.loop._run_once()
        self.assertTrue(self.sock.close.called)
        tr.close()

    def test_write_eof_duplex_pipe(self):
        tr = _ProactorDuplexPipeTransport(self.loop, self.sock, self.protocol)
        self.assertFalse(tr.can_write_eof())
        with self.assertRaises(NotImplementedError):
            tr.write_eof()
        close_transport(tr)

    def test_pause_resume_reading(self):
        tr = self.socket_transport()
        futures = []
        for msg in [b'data1', b'data2', b'data3', b'data4', b'']:
            f = asyncio.Future(loop=self.loop)
            f.set_result(msg)
            futures.append(f)
        self.loop._proactor.recv.side_effect = futures
        self.loop._run_once()
        self.assertFalse(tr._paused)
        self.loop._run_once()
        self.protocol.data_received.assert_called_with(b'data1')
        self.loop._run_once()
        self.protocol.data_received.assert_called_with(b'data2')
        tr.pause_reading()
        self.assertTrue(tr._paused)
        for i in range(10):
            self.loop._run_once()
        self.protocol.data_received.assert_called_with(b'data2')
        tr.resume_reading()
        self.assertFalse(tr._paused)
        self.loop._run_once()
        self.protocol.data_received.assert_called_with(b'data3')
        self.loop._run_once()
        self.protocol.data_received.assert_called_with(b'data4')
        tr.close()

    def pause_writing_transport(self, high):
        tr = self.socket_transport()
        tr.set_write_buffer_limits(high=high)

        self.assertEqual(tr.get_write_buffer_size(), 0)
        self.assertFalse(self.protocol.pause_writing.called)
        self.assertFalse(self.protocol.resume_writing.called)
        return tr

    def test_pause_resume_writing(self):
        tr = self.pause_writing_transport(high=4)

        # write a large chunk, must pause writing
        fut = asyncio.Future(loop=self.loop)
        self.loop._proactor.send.return_value = fut
        tr.write(b'large data')
        self.loop._run_once()
        self.assertTrue(self.protocol.pause_writing.called)

        # flush the buffer
        fut.set_result(None)
        self.loop._run_once()
        self.assertEqual(tr.get_write_buffer_size(), 0)
        self.assertTrue(self.protocol.resume_writing.called)

    def test_pause_writing_2write(self):
        tr = self.pause_writing_transport(high=4)

        # first short write, the buffer is not full (3 <= 4)
        fut1 = asyncio.Future(loop=self.loop)
        self.loop._proactor.send.return_value = fut1
        tr.write(b'123')
        self.loop._run_once()
        self.assertEqual(tr.get_write_buffer_size(), 3)
        self.assertFalse(self.protocol.pause_writing.called)

        # fill the buffer, must pause writing (6 > 4)
        tr.write(b'abc')
        self.loop._run_once()
        self.assertEqual(tr.get_write_buffer_size(), 6)
        self.assertTrue(self.protocol.pause_writing.called)

    def test_pause_writing_3write(self):
        tr = self.pause_writing_transport(high=4)

        # first short write, the buffer is not full (1 <= 4)
        fut = asyncio.Future(loop=self.loop)
        self.loop._proactor.send.return_value = fut
        tr.write(b'1')
        self.loop._run_once()
        self.assertEqual(tr.get_write_buffer_size(), 1)
        self.assertFalse(self.protocol.pause_writing.called)

        # second short write, the buffer is not full (3 <= 4)
        tr.write(b'23')
        self.loop._run_once()
        self.assertEqual(tr.get_write_buffer_size(), 3)
        self.assertFalse(self.protocol.pause_writing.called)

        # fill the buffer, must pause writing (6 > 4)
        tr.write(b'abc')
        self.loop._run_once()
        self.assertEqual(tr.get_write_buffer_size(), 6)
        self.assertTrue(self.protocol.pause_writing.called)

    def test_dont_pause_writing(self):
        tr = self.pause_writing_transport(high=4)

        # write a large chunk which completes immedialty,
        # it should not pause writing
        fut = asyncio.Future(loop=self.loop)
        fut.set_result(None)
        self.loop._proactor.send.return_value = fut
        tr.write(b'very large data')
        self.loop._run_once()
        self.assertEqual(tr.get_write_buffer_size(), 0)
        self.assertFalse(self.protocol.pause_writing.called)


class BaseProactorEventLoopTests(test_utils.TestCase):
    def setUp(self):
        super().setUp()

        self.sock = test_utils.mock_nonblocking_socket()
        self.proactor = mock.Mock()

        self.ssock, self.csock = mock.Mock(), mock.Mock()

        class EventLoop(BaseProactorEventLoop):
            def _socketpair(s):
                return (self.ssock, self.csock)

        self.loop = EventLoop(self.proactor)
        self.set_event_loop(self.loop)

    @unittest.skipIf(sys.version_info >= (3, 7), 'TODO breaks')
    @mock.patch.object(BaseProactorEventLoop, 'call_soon')
    @mock.patch.object(BaseProactorEventLoop, '_socketpair')
    def test_ctor(self, socketpair, call_soon):
        ssock, csock = socketpair.return_value = (mock.Mock(), mock.Mock())
        loop = BaseProactorEventLoop(self.proactor)
        self.assertIs(loop._ssock, ssock)
        self.assertIs(loop._csock, csock)
        self.assertEqual(loop._internal_fds, 1)
        call_soon.assert_called_with(loop._loop_self_reading)
        loop.close()

    @unittest.skipIf(sys.version_info >= (3, 7), 'TODO breaks')
    def test_close_self_pipe(self):
        self.loop._close_self_pipe()
        self.assertEqual(self.loop._internal_fds, 0)
        self.assertTrue(self.ssock.close.called)
        self.assertTrue(self.csock.close.called)
        self.assertIsNone(self.loop._ssock)
        self.assertIsNone(self.loop._csock)

        # Don't call close(): _close_self_pipe() cannot be called twice
        self.loop._closed = True

    def test_close(self):
        self.loop._close_self_pipe = mock.Mock()
        self.loop.close()
        self.assertTrue(self.loop._close_self_pipe.called)
        self.assertTrue(self.proactor.close.called)
        self.assertIsNone(self.loop._proactor)

        self.loop._close_self_pipe.reset_mock()
        self.loop.close()
        self.assertFalse(self.loop._close_self_pipe.called)

    @unittest.skipIf(sys.version_info >= (3, 7), 'XXX does not work')
    def test_sock_recv(self):
        self.loop.sock_recv(self.sock, 1024)
        test_utils.run_briefly(self.loop)
        assert self.proactor.recv._mock_call_args_list[0][0][0] is self.sock

    @unittest.skipIf(sys.version_info >= (3, 7), 'XXX does not work')
    def test_sock_sendall(self):
        self.loop.sock_sendall(self.sock, b'data')
        test_utils.run_briefly(self.loop)
        self.proactor.send.assert_called_with(self.sock, b'data')

    @unittest.skipIf(sys.version_info >= (3, 7), 'XXX does not work')
    def test_sock_connect(self):
        self.loop.sock_connect(self.sock, ('1.2.3.4', 123))
        test_utils.run_briefly(self.loop)
        self.proactor.connect.assert_called_with(self.sock, ('1.2.3.4', 123))

    @unittest.skipIf(sys.version_info >= (3, 7), 'XXX does not work')
    def test_sock_accept(self):
        self.loop.sock_accept(self.sock)
        test_utils.run_briefly(self.loop)
        self.proactor.accept.assert_called_with(self.sock)

    @unittest.skipIf(sys.version_info >= (3, 7), 'deleted')
    def test_socketpair(self):
        class EventLoop(BaseProactorEventLoop):
            # override the destructor to not log a ResourceWarning
            def __del__(self):
                pass

        self.assertRaises(NotImplementedError, EventLoop, self.proactor)

    def test_make_socket_transport(self):
        tr = self.loop._make_socket_transport(self.sock, asyncio.Protocol())
        self.assertIsInstance(tr, _ProactorSocketTransport)
        close_transport(tr)

    @unittest.skipIf(sys.version_info >= (3, 7), 'TODO breaks')
    def test_loop_self_reading(self):
        self.loop._loop_self_reading()
        self.proactor.recv.assert_called_with(self.ssock, 4096)
        self.proactor.recv.return_value.add_done_callback.assert_called_with(
            self.loop._loop_self_reading
        )

    @unittest.skipIf(sys.version_info >= (3, 7), 'TODO breaks')
    def test_loop_self_reading_fut(self):
        fut = mock.Mock()
        self.loop._loop_self_reading(fut)
        self.assertTrue(fut.result.called)
        self.proactor.recv.assert_called_with(self.ssock, 4096)
        self.proactor.recv.return_value.add_done_callback.assert_called_with(
            self.loop._loop_self_reading
        )

    def test_loop_self_reading_exception(self):
        self.loop.call_exception_handler = mock.Mock()
        self.proactor.recv.side_effect = OSError()
        self.loop._loop_self_reading()
        self.assertTrue(self.loop.call_exception_handler.called)

    @unittest.skipIf(sys.version_info >= (3, 7), 'XXX does not work')
    def test_write_to_self(self):
        self.loop._write_to_self()
        test_utils.run_briefly(self.loop)
        self.csock.send.assert_called_with(b'\0')

    def test_process_events(self):
        self.loop._process_events([])

    @mock.patch('asyncio.base_events.logger')
    def test_create_server(self, m_log):
        pf = mock.Mock()
        call_soon = self.loop.call_soon = mock.Mock()

        self.loop._start_serving(pf, self.sock)
        self.assertTrue(call_soon.called)

        # callback
        loop = call_soon.call_args[0][0]
        loop()
        self.proactor.accept.assert_called_with(self.sock)

        # conn
        fut = mock.Mock()
        fut.result.return_value = (mock.Mock(), mock.Mock())

        make_tr = self.loop._make_socket_transport = mock.Mock()
        loop(fut)
        self.assertTrue(fut.result.called)
        self.assertTrue(make_tr.called)

        # exception
        fut.result.side_effect = OSError()
        loop(fut)
        self.assertTrue(self.sock.close.called)
        self.assertTrue(m_log.error.called)

    def test_create_server_cancel(self):
        pf = mock.Mock()
        call_soon = self.loop.call_soon = mock.Mock()

        self.loop._start_serving(pf, self.sock)
        loop = call_soon.call_args[0][0]

        # cancelled
        fut = asyncio.Future(loop=self.loop)
        fut.cancel()
        loop(fut)
        self.assertTrue(self.sock.close.called)

    def test_stop_serving(self):
        sock = mock.Mock()
        self.loop._stop_serving(sock)
        self.assertTrue(sock.close.called)
        self.proactor._stop_serving.assert_called_with(sock)


if __name__ == '__main__':
    unittest.main()



import sys
import trio
import asyncio
try:
    from asyncio.format_helpers import _format_callback, _get_function_source
except ImportError:  # <3.7
    from asyncio.events import _format_callback, _get_function_source

__all__ = ['Handle', 'TimerHandle']


def _format_callback_source(func, args):
    func_repr = _format_callback(func, args, None)
    source = _get_function_source(func)
    if source:  # pragma: no cover
        func_repr += ' at %s:%s' % source
    return func_repr


class _TrioHandle:
    """
    This extends asyncio.Handle by providing:
    * a way to cancel an async callback
    * a way to declare the type of the callback function

    ``is_sync`` may be
    * True: sync function, use _call_sync()
    * False: async function, use _call_async()
    * None: also async, but the callback function accepts
      the handle as its sole argument

    The caller is responsible for checking whether the handle
    has been cancelled before invoking ``call_[a]sync()``.
    """

    def _init(self, is_sync):
        """Secondary init.
        """
        self._is_sync = is_sync
        self._scope = None

    def cancel(self):
        super().cancel()
        if self._scope is not None:
            self._scope.cancel()

    def _cb_future_cancel(self, f):
        """If a Trio task completes an asyncio Future,
        add this callback to the future
        and set ``_scope`` to the Trio cancel scope
        so that the task is terminated when the future gets canceled.

        """
        if f.cancelled():
            self.cancel()

    def _raise(self, exc):
        """This is a copy of the exception handling in asyncio.events.Handle._run()
        """
        cb = _format_callback_source(self._callback, self._args)
        msg = 'Exception in callback {}'.format(cb)
        context = {
            'message': msg,
            'exception': exc,
            'handle': self,
        }
        if self._source_traceback:
            context['source_traceback'] = self._source_traceback
        self._loop.call_exception_handler(context)

    def _repr_info(self):
        info = [self.__class__.__name__]
        if self._cancelled:
            info.append('cancelled')
        if self._callback is not None:
            info.append(_format_callback_source(self._callback, self._args))
        if self._source_traceback:
            frame = self._source_traceback[-1]
            info.append('created at %s:%s' % (frame[0], frame[1]))
        if self._scope is not None:
            info.append('scope=%s' % repr(self._scope))
        return info

    def _call_sync(self):
        assert self._is_sync
        if self._cancelled:
            return
        self._run()

    if sys.version_info >= (3, 7):

        async def _call_async(self, task_status=trio.TASK_STATUS_IGNORED):
            assert not self._is_sync
            if self._cancelled:
                return
            task_status.started()
            try:
                with trio.open_cancel_scope() as scope:
                    self._scope = scope
                    if self._is_sync is None:
                        await self._context.run(self._callback, self)
                    else:
                        await self._context.run(self._callback, *self._args)
            except Exception as exc:
                self._raise(exc)
            finally:
                self._scope = None

    else:  # no contextvars

        async def _call_async(self, task_status=trio.TASK_STATUS_IGNORED):
            assert not self._is_sync
            if self._cancelled:
                return
            task_status.started()
            try:
                with trio.open_cancel_scope() as scope:
                    self._scope = scope
                    if self._is_sync is None:
                        await self._callback(self)
                    else:
                        await self._callback(*self._args)
            except Exception as exc:
                self._raise(exc)
            finally:
                self._scope = None


if sys.version_info >= (3, 7):

    class Handle(_TrioHandle, asyncio.Handle):
        def __init__(self, callback, args, loop, context=None, is_sync=True):
            assert not isinstance(context, bool)
            super().__init__(callback, args, loop, context=context)
            self._init(is_sync)

    class TimerHandle(_TrioHandle, asyncio.TimerHandle):
        def __init__(self, when, callback, args, loop, context=None, is_sync=True):
            assert not isinstance(context, bool)
            super().__init__(when, callback, args, loop, context=context)
            self._init(is_sync)

else:

    class Handle(_TrioHandle, asyncio.Handle):
        def __init__(self, callback, args, loop, context=None, is_sync=True):
            super().__init__(callback, args, loop)
            self._init(is_sync)

    class TimerHandle(_TrioHandle, asyncio.TimerHandle):
        def __init__(self, when, callback, args, loop, context=None, is_sync=True):
            super().__init__(when, callback, args, loop)
            self._init(is_sync)



# This code implements basic asyncio compatibility

from ._version import __version__  # noqa

from .base import *  # noqa
from .loop import *  # noqa
from .util import *  # noqa
from .async_ import *  # noqa
from .adapter import *  # noqa



import os
import sys
import threading
import weakref
import outcome

import trio

import logging
logger = logging.getLogger(__name__)

_mswindows = (sys.platform == "win32")
if _mswindows:
    import _winapi

__all__ = ['wait_for_child']

# TODO: use whatever works for Windows and MacOS/BSD

_children = weakref.WeakValueDictionary()


class UnknownStatus(ChildProcessError):
    pass


def _compute_returncode(status):
    if os.WIFSIGNALED(status):
        # The child process died because of a signal.
        return -os.WTERMSIG(status)
    elif os.WIFEXITED(status):
        # The child process exited (e.g sys.exit()).
        return os.WEXITSTATUS(status)
    elif os.WIFSTOPPED(status):
        return -os.WSTOPSIG(status)
    else:
        # This shouldn't happen.
        raise UnknownStatus(status)


def NOT_FOUND():
    return outcome.Error(ChildProcessError())


class ProcessWaiter:
    """I implement waiting for a child process."""

    __token = None
    __event = None
    __pid = None
    __result = None
    __thread = None
    _handle = None

    def __new__(cls, pid=None, _handle=None):
        """Grab an existing object if there is one"""
        self = None
        if pid is not None:
            self = _children.get(pid, None)
        if self is None:
            self = object.__new__(cls)
        return self

    def __init__(self, pid=None, _handle=None):
        if self.__pid is None:
            self._set_pid(pid, _handle)

    def _set_pid(self, pid, _handle=None):
        if self.__pid is not None:
            raise RuntimeError("You can't change the pid")
        if not isinstance(pid, int):
            raise RuntimeError("a PID needs to be an integer")

        self.__pid = pid
        _children[pid] = self

        if _mswindows:
            if _handle is None:
                _handle = _winapi.OpenProcess(_winapi.PROCESS_ALL_ACCESS, True, pid)
            self.__handle = _handle
        elif _handle is not None:
            raise RuntimeError("Process handles are a Windows thing.")

    async def wait(self):
        """Wait for this child process to end."""
        if self.__result is None:
            if self.__pid is None:
                raise RuntimeError("I don't know what to wait for!")

            # Check once, before doing the heavy lifting
            self._wait_pid(blocking=False)
            if self.__result is None:
                if self.__thread is None:
                    await self._start_waiting()
                await self.__event.wait()
        return self.__result.unwrap()

    async def _start_waiting(self):
        """Start the background thread that waits for a specific child"""
        self.__event = trio.Event()
        self.__token = trio.hazmat.current_trio_token()

        self.__thread = threading.Thread(
            target=self._wait_thread, name="waitpid_%d" % self.__pid, daemon=True
        )
        self.__thread.start()

    def _wait_thread(self):
        """The background thread that waits for a specific child"""
        self._wait_pid(blocking=True)
        self.__token.run_sync_soon(self.__event.set)

    if _mswindows:

        def _wait_pid(self, blocking):
            assert self.__handle is not None
            if blocking:
                timeout = _winapi.INFINITE
            else:
                timeout = 0
            result = _winapi.WaitForSingleObject(self.__handle, timeout)
            if result != _winapi.WAIT_TIMEOUT:
                self.__result = _winapi.GetExitCodeProcess(self._handle)

    else:

        def _wait_pid(self, blocking):
            """check up on a child process"""
            assert self.__pid > 0

            try:
                pid, status = os.waitpid(self.__pid, 0 if blocking else os.WNOHANG)
            except ChildProcessError:
                # The child process may already be reaped
                # (may happen if waitpid() is called elsewhere).
                self.__result = NOT_FOUND()
            else:
                if pid == 0:
                    # The child process is still alive.
                    return
                del _children[pid]
                self._handle_exitstatus(status)

        def _handle_exitstatus(self, sts):
            """This overrides an internal API of subprocess.Popen"""
            self.__result = outcome.capture(_compute_returncode, sts)

    @property
    def returncode(self):
        if self.__result is None:
            return None
        return self.__result.unwrap()


async def wait_for_child(pid):
    waiter = ProcessWaiter(pid)
    return await waiter.wait()



from tests import aiotest
import signal
import pytest


class TestCallback(aiotest.TestCase):
    @pytest.mark.trio
    async def test_call_soon(self, loop):
        result = []

        def hello_world(loop):
            result.append('Hello World')
            loop.stop()

        loop.call_soon(hello_world, loop)
        await loop.stop().wait()
        assert result == ['Hello World']

    @pytest.mark.trio
    async def test_call_soon_control(self, loop):
        result = []

        def func(result, loop):
            loop.call_soon(append_result, loop, result, "yes")
            result.append(str(result))

        def append_result(loop, result, value):
            result.append(value)
            loop.stop()

        loop.call_soon(func, result, loop)
        await loop.wait_stopped()
        # http://bugs.python.org/issue22875: Ensure that call_soon() does not
        # call append_result() immediatly, but when control returns to the
        # event loop, when func() is done.
        assert result == ['[]', 'yes']

    def test_soon_stop_soon(self, sync_loop):
        result = []

        def hello():
            result.append("Hello")

        def world():
            result.append("World")
            sync_loop.stop()

        sync_loop.call_soon(hello)
        sync_loop.stop()
        sync_loop.call_soon(world)

        sync_loop.run_forever()
        if False:  # config.stopping:
            assert result == ["Hello", "World"]
        else:
            # ensure that world() is not called, since stop() was scheduled
            # before call_soon(world)
            assert result == ["Hello"]

            sync_loop.run_forever()
            assert result == ["Hello", "World"]

    @pytest.mark.trio
    async def test_close(self, loop, config):
        if not config.call_soon_check_closed:
            # http://bugs.python.org/issue22922 not implemented
            self.skipTest("call_soon() doesn't raise if the event loop is closed")

        await loop.stop().wait()
        loop.close()

        @config.asyncio.coroutine
        def test():
            pass

        func = lambda: False
        coro = test()
        try:
            # no longer depends on the loop
            # with pytest.raises(RuntimeError):
            #    fut = config.asyncio.Future(loop=loop)
            #    await loop.run_future(fut)
            with pytest.raises(RuntimeError, match='not a sync loop'):
                loop.run_until_complete(None)
            with pytest.raises(RuntimeError):
                loop.run_forever()
            with pytest.raises(RuntimeError, match='Event loop is closed'):
                loop.call_soon(func)
            with pytest.raises(RuntimeError, match='Event loop is closed'):
                loop.call_soon_threadsafe(func)
            with pytest.raises(RuntimeError, match='Event loop is closed'):
                loop.call_later(1.0, func)
            with pytest.raises(RuntimeError, match='Event loop is closed'):
                loop.call_at(loop.time() + .0, func)
            with pytest.raises(RuntimeError, match='Event loop is closed'):
                loop.run_in_executor(None, func)
            with pytest.raises(RuntimeError, match='Event loop is closed'):
                await loop.run_coroutine(coro)
            with pytest.raises(RuntimeError, match='Event loop is closed'):
                loop.add_signal_handler(signal.SIGTERM, func)
        finally:
            coro.close()



from tests import aiotest
import pytest


def create_classes(config):
    asyncio = config.asyncio
    socket = config.socket
    threading = config.threading

    class TcpEchoClientProtocol(asyncio.Protocol):
        def __init__(self, message, loop):
            self.message = message
            self.loop = loop
            self.state = 'new'
            self.received = None

        def connection_made(self, transport):
            self.state = 'ping'
            transport.write(self.message)

        def data_received(self, data):
            self.state = 'pong'
            self.received = data

        def connection_lost(self, exc):
            self.state = 'closed'
            self.loop.stop()

    class TcpServer(threading.Thread):
        def __init__(self, host, port, event):
            super(TcpServer, self).__init__()
            self.host = host
            self.port = port
            self.event = event
            self.sock = None
            self.client = None

        def run(self):
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self.sock = sock
            try:
                sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                sock.bind((self.host, self.port))
                sock.listen(1)

                self.event.set()
                client, addr = sock.accept()
                self.client = client
                try:
                    message = client.recv(100)
                    client.sendall(message)
                finally:
                    client.close()
                    self.client = None
            finally:
                sock.close()
                self.sock = None

        def stop(self):
            self.join()

    return TcpEchoClientProtocol, TcpServer


class TestNetwork(aiotest.TestCase):
    @pytest.mark.trio
    async def test_tcp_hello(self, loop, config):
        return
        port = 8888
        host = '127.0.0.1'
        message = b'Hello World!'

        event = config.threading.Event()
        TcpEchoClientProtocol, TcpServer = create_classes(config)
        server = TcpServer(host, port, event)
        server.start()
        self.addCleanup(server.stop)
        event.wait()

        proto = TcpEchoClientProtocol(message, loop)
        coro = loop.create_connection(lambda: proto, host, port)
        await loop.run_coroutine(coro)
        assert proto.state != 'new'

        await loop.stop().wait()()
        assert proto.state == 'closed'
        assert proto.received == message



from tests import aiotest
import pytest


async def hello_world(asyncio, result, delay, loop):
    result.append("Hello")
    # retrieve the event loop from the policy
    await asyncio.sleep(delay, loop=loop)
    result.append('World')
    return "."


class TestCoroutine(aiotest.TestCase):
    @pytest.mark.trio
    async def test_hello_world(self, loop, config):
        result = []
        coro = hello_world(config.asyncio, result, 0.001, loop)
        await loop.run_coroutine(config.asyncio.ensure_future(coro, loop=loop))
        assert result == ['Hello', 'World']

    @pytest.mark.trio
    async def run_hello_world(self, loop, config):
        result = []
        await loop.run_asyncio(hello_world, config.asyncio, result, 0.001, loop)
        assert result == ['Hello', 'World']

    @pytest.mark.trio
    async def test_waiter(self, loop, config):
        async def waiter(asyncio, hello_world, result):
            fut = asyncio.Future(loop=loop)
            loop.call_soon(fut.set_result, "Future")

            value = await fut
            result.append(value)

            value = await hello_world(asyncio, result, 0.001, loop)
            result.append(value)

        result = []
        await loop.run_asyncio(waiter, config.asyncio, hello_world, result)
        assert result == ['Future', 'Hello', 'World', '.']



"""Tests for asyncio/sslproto.py."""

import logging
import unittest
from unittest import mock
import sys
try:
    import ssl
except ImportError:
    ssl = None

import asyncio
from asyncio import log
from asyncio import sslproto
from .. import utils as test_utils


@unittest.skipIf(ssl is None, 'No ssl module')
class SslProtoHandshakeTests(test_utils.TestCase):
    def setUp(self):
        super().setUp()
        self.loop = asyncio.new_event_loop()
        self.set_event_loop(self.loop)

    def ssl_protocol(self, waiter=None):
        sslcontext = test_utils.dummy_ssl_context()
        app_proto = asyncio.Protocol()
        proto = sslproto.SSLProtocol(self.loop, app_proto, sslcontext, waiter)
        self.assertIs(proto._app_transport.get_protocol(), app_proto)
        self.addCleanup(proto._app_transport.close)
        return proto

    def connection_made(self, ssl_proto, do_handshake=None):
        transport = mock.Mock()
        sslpipe = mock.Mock()
        sslpipe.shutdown.return_value = b''
        if do_handshake:
            sslpipe.do_handshake.side_effect = do_handshake
        else:

            def mock_handshake(callback):
                return []

            sslpipe.do_handshake.side_effect = mock_handshake
        with mock.patch('asyncio.sslproto._SSLPipe', return_value=sslpipe):
            ssl_proto.connection_made(transport)
        return transport

    def test_cancel_handshake(self):
        # Python issue #23197: cancelling a handshake must not raise an
        # exception or log an error, even if the handshake failed
        waiter = asyncio.Future(loop=self.loop)
        ssl_proto = self.ssl_protocol(waiter)
        handshake_fut = asyncio.Future(loop=self.loop)

        def do_handshake(callback):
            exc = Exception()
            callback(exc)
            handshake_fut.set_result(None)
            return []

        waiter.cancel()
        self.connection_made(ssl_proto, do_handshake)

        with test_utils.disable_logger():
            self.loop.run_until_complete(handshake_fut)

    def test_eof_received_waiter(self):
        waiter = asyncio.Future(loop=self.loop)
        ssl_proto = self.ssl_protocol(waiter)
        self.connection_made(ssl_proto)
        ssl_proto.eof_received()
        test_utils.run_briefly(self.loop)
        self.assertIsInstance(waiter.exception(), ConnectionResetError)

    def test_fatal_error_no_name_error(self):
        # From issue #363.
        # _fatal_error() generates a NameError if sslproto.py
        # does not import base_events.
        waiter = asyncio.Future(loop=self.loop)
        ssl_proto = self.ssl_protocol(waiter)
        # Temporarily turn off error logging so as not to spoil test output.
        log_level = log.logger.getEffectiveLevel()
        log.logger.setLevel(logging.FATAL)
        try:
            ssl_proto._fatal_error(None)
        finally:
            # Restore error logging.
            log.logger.setLevel(log_level)

    def test_connection_lost(self):
        # From issue #472.
        # yield from waiter hang if lost_connection was called.
        waiter = asyncio.Future(loop=self.loop)
        ssl_proto = self.ssl_protocol(waiter)
        self.connection_made(ssl_proto)
        ssl_proto.connection_lost(ConnectionAbortedError)
        test_utils.run_briefly(self.loop)
        self.assertIsInstance(waiter.exception(), ConnectionAbortedError)

    @unittest.skipIf(sys.version_info < (3, 6), "Fixed in 3.6")
    def test_close_during_handshake(self):
        # bpo-29743 Closing transport during handshake process leaks socket
        waiter = asyncio.Future(loop=self.loop)
        ssl_proto = self.ssl_protocol(waiter)

        def do_handshake(callback):
            return []

        transport = self.connection_made(ssl_proto)
        test_utils.run_briefly(self.loop)

        ssl_proto._app_transport.close()
        self.assertTrue(transport.abort.called)

    @unittest.skipIf(sys.version_info < (3, 6), "Fixed in 3.6")
    def test_get_extra_info_on_closed_connection(self):
        waiter = asyncio.Future(loop=self.loop)
        ssl_proto = self.ssl_protocol(waiter)
        self.assertIsNone(ssl_proto._get_extra_info('socket'))
        default = object()
        self.assertIs(ssl_proto._get_extra_info('socket', default), default)
        self.connection_made(ssl_proto)
        self.assertIsNotNone(ssl_proto._get_extra_info('socket'))
        ssl_proto.connection_lost(None)
        self.assertIsNone(ssl_proto._get_extra_info('socket'))

    def test_set_new_app_protocol(self):
        waiter = asyncio.Future(loop=self.loop)
        ssl_proto = self.ssl_protocol(waiter)
        new_app_proto = asyncio.Protocol()
        ssl_proto._app_transport.set_protocol(new_app_proto)
        self.assertIs(ssl_proto._app_transport.get_protocol(), new_app_proto)
        if sys.version_info >= (3, 6, 4):
            self.assertIs(ssl_proto._app_protocol, new_app_proto)


if __name__ == '__main__':
    unittest.main()



# This code implements a clone of the asyncio mainloop which hooks into
# Trio.

from async_generator import isasyncgenfunction
import trio_asyncio

# import logging
# logger = logging.getLogger(__name__)

from functools import wraps, partial

__all__ = ['trio2aio', 'aio2trio']


def trio2aio(proc):
    if isasyncgenfunction(proc):

        @wraps(proc)
        def call(*args, **kwargs):
            proc_ = proc
            if kwargs:
                proc_ = partial(proc_, **kwargs)
            return trio_asyncio.wrap_generator(proc_, *args)

    else:

        @wraps(proc)
        async def call(*args, **kwargs):
            proc_ = proc
            if kwargs:
                proc_ = partial(proc_, **kwargs)
            return await trio_asyncio.run_asyncio(proc_, *args)

    return call


def aio2trio(proc):
    """Decorate a Trio function so that it's callable by asyncio (only)."""

    @wraps(proc)
    async def call(*args, **kwargs):
        proc_ = proc
        if kwargs:
            proc_ = partial(proc_, **kwargs)
        return await trio_asyncio.run_trio(proc_, *args)

    return call


def aio2trio_task(proc):
    @wraps(proc)
    async def call(*args, **kwargs):
        proc_ = proc
        if kwargs:
            proc_ = partial(proc_, **kwargs)
        trio_asyncio.run_trio_task(proc_, *args)

    return call



# This code implements helper functions that work without running
# a TrioEventLoop.

import trio
import asyncio
import sys
import outcome
from async_generator import async_generator, yield_

__all__ = ['run_future']


async def run_future(future):
    """Wait for an asyncio future/coroutine from Trio code.

    Cancelling the current Trio scope will cancel the future/coroutine.

    Cancelling the future/coroutine will cause an ``asyncio.CancelledError``.
    """
    task = trio.hazmat.current_task()
    raise_cancel = None

    def done_cb(_):
        trio.hazmat.reschedule(task, outcome.capture(future.result))

    future.add_done_callback(done_cb)

    def abort_cb(raise_cancel_arg):
        # Save the cancel-raising function
        nonlocal raise_cancel
        raise_cancel = raise_cancel_arg
        # Attempt to cancel our future
        future.cancel()
        # Keep waiting
        return trio.hazmat.Abort.FAILED

    try:
        res = await trio.hazmat.wait_task_rescheduled(abort_cb)
        return res
    except asyncio.CancelledError as exc:
        if raise_cancel is not None:
            try:
                raise_cancel()
            finally:
                # Try to preserve the exception chain,
                # for more detailed tracebacks
                sys.exc_info()[1].__cause__ = exc
        else:
            raise


STOP = object()


@async_generator
async def run_generator(loop, async_generator):
    task = trio.hazmat.current_task()
    raise_cancel = None
    current_read = None

    async def consume_next():
        try:
            item = await async_generator.__anext__()
            result = outcome.Value(value=item)
        except StopAsyncIteration:
            result = outcome.Value(value=STOP)
        except asyncio.CancelledError:
            # Once we are cancelled, we do not call reschedule() anymore
            return
        except Exception as e:
            result = outcome.Error(error=e)

        trio.hazmat.reschedule(task, result)

    def abort_cb(raise_cancel_arg):
        # Save the cancel-raising function
        nonlocal raise_cancel
        raise_cancel = raise_cancel_arg

        if not current_read:
            # There is no current read
            return trio.hazmat.Abort.SUCCEEDED
        else:
            # Attempt to cancel the current iterator read, do not
            # report success until the future was actually cancelled.
            already_cancelled = current_read.cancel()
            if already_cancelled:
                return trio.hazmat.Abort.SUCCEEDED
            else:
                # Continue dealing with the cancellation once
                # future.cancel() goes to the result of
                # wait_task_rescheduled()
                return trio.hazmat.Abort.FAILED

    try:
        while True:
            # Schedule in asyncio that we read the next item from the iterator
            current_read = asyncio.ensure_future(consume_next(), loop=loop)

            item = await trio.hazmat.wait_task_rescheduled(abort_cb)

            if item is STOP:
                break
            await yield_(item)

    except asyncio.CancelledError as exc:
        if raise_cancel is not None:
            try:
                raise_cancel()
            finally:
                # Try to preserve the exception chain,
                # for more detailed tracebacks
                sys.exc_info()[1].__cause__ = exc
        else:
            raise



import os

if __name__ == '__main__':
    while True:
        buf = os.read(0, 1024)
        if not buf:
            break
        try:
            os.write(1, b'OUT:' + buf)
        except OSError as ex:
            os.write(2, b'ERR:' + ex.__class__.__name__.encode('ascii'))



import socket
import sys
import threading
import time

import asyncio as _asyncio
if sys.platform == 'win32':
    from asyncio.windows_utils import socketpair
else:
    socketpair = socket.socketpair


class TestConfig:
    def __init__(self):
        # Stop on first fail or error
        self.fail_fast = False

        # Run tests forever to catch sporadic errors
        self.forever = False

        # Verbosity 0..4: 0=less messages (CRITICAL), 4=more messages (DEBUG)
        self.verbosity = 0

        # List of test names to include, empty list means that all tests
        # are included
        self.includes = []

        # List of test names to exclude, empty list means that no test is
        # excluded
        self.excludes = []

        # modules
        self.asyncio = _asyncio
        self.socket = socket
        self.threading = threading

        # functions
        self.socketpair = socketpair
        self.sleep = time.sleep

        # features of the implementations

        # The event loop can be run in a thread different than the main thread?
        self.support_threads = True

        # http://bugs.python.org/issue22922
        # call_soon() now raises an exception when the event loop is closed
        self.call_soon_check_closed = True

        # http://bugs.python.org/issue25593
        # Change semantics of EventLoop.stop(). Replace _StopError exception
        # with a new stopping attribute.
        self.stopping = True

    def prepare(self, testcase):
        # import pdb;pdb.set_trace()
        # policy = self.new_event_pool_policy()
        # self.asyncio.set_event_loop_policy(policy)
        testcase.addCleanup(self.asyncio.set_event_loop_policy, None)

        testcase.loop = self.asyncio.get_event_loop()
        # testcase.addCleanup(testcase.loop.close)
        # testcase.addCleanup(self.asyncio.set_event_loop, None)


class TestCase:
    pass


#    @classmethod
#    def setUpClass(cls):
#        cls.config = config
#
#    def setUp(self):
#        self.config.prepare(self)
